@@ -84,7 +84,9 @@ test_virtualenvwrapper_verify_workon_home_missing_dir_grep_options() {
 test_python_interpreter_set_incorrectly() {
     return_to="$(pwd)"
     cd "$WORKON_HOME"
-    mkvirtualenv no_wrappers
+    mkvirtualenv no_wrappers >/dev/null 2>&1
+	RC=$?
+	assertEquals "0" "$RC"
     expected="ImportError: No module named virtualenvwrapper.hook_loader"
     # test_shell is set by tests/run_tests
     if [ "$test_shell" = "" ]
@@ -102,4 +104,18 @@ test_python_interpreter_set_incorrectly() {
     deactivate
 }
 
+test_virtualenvwrapper_verify_virtualenv(){
+    assertTrue "Verified unable to verify virtualenv" virtualenvwrapper_verify_virtualenv
+
+    VIRTUALENVWRAPPER_VIRTUALENV="thiscannotpossiblyexist123"
+    assertFalse "Incorrectly verified virtualenv" virtualenvwrapper_verify_virtualenv
+}
+
+test_virtualenvwrapper_verify_virtualenv_clone(){
+    assertTrue "Verified unable to verify virtualenv_clone" virtualenvwrapper_verify_virtualenv_clone
+
+    VIRTUALENVWRAPPER_VIRTUALENV_CLONE="thiscannotpossiblyexist123"
+    assertFalse "Incorrectly verified virtualenv_clone" virtualenvwrapper_verify_virtualenv_clone
+}
+
 . "$test_dir/shunit2"
