@@ -47,7 +47,7 @@
 # Locate the global Python where virtualenvwrapper is installed.
 if [ "$VIRTUALENVWRAPPER_PYTHON" = "" ]
 then
-    VIRTUALENVWRAPPER_PYTHON="$(\which python)"
+    VIRTUALENVWRAPPER_PYTHON="$(command \which python)"
 fi
 
 # Set the name of the virtualenv app to use.
@@ -56,6 +56,12 @@ then
     VIRTUALENVWRAPPER_VIRTUALENV="virtualenv"
 fi
 
+# Set the name of the virtualenv-clone app to use.
+if [ "$VIRTUALENVWRAPPER_VIRTUALENV_CLONE" = "" ]
+then
+    VIRTUALENVWRAPPER_VIRTUALENV_CLONE="virtualenv-clone"
+fi
+
 # Define script folder depending on the platorm (Win32/Unix)
 VIRTUALENVWRAPPER_ENV_BIN_DIR="bin"
 if [ "$OS" = "Windows_NT" ] && [ "$MSYSTEM" = "MINGW32" ]
@@ -72,13 +78,36 @@ then
     export VIRTUALENVWRAPPER_PROJECT_FILENAME=".project"
 fi
 
+# Portable shell scripting is hard, let's go shopping.
+#
+# People insist on aliasing commands like 'cd', either with a real
+# alias or even a shell function. Under bash and zsh, "builtin" forces
+# the use of a command that is part of the shell itself instead of an
+# alias, function, or external command, while "command" does something
+# similar but allows external commands. Under ksh "builtin" registers
+# a new command from a shared library, but "command" will pick up
+# existing builtin commands. We need to use a builtin for cd because
+# we are trying to change the state of the current shell, so we use
+# "builtin" for bash and zsh but "command" under ksh.
+function virtualenvwrapper_cd {
+    if [ -n "$BASH" ]
+    then
+        builtin \cd "$@"
+    elif [ -n "$ZSH_VERSION" ]
+    then
+        builtin \cd "$@"
+    else
+        command \cd "$@"
+    fi
+}
+
 function virtualenvwrapper_expandpath {
-        if [ "$1" = "" ]; then
-            return 1
-        else
-            "$VIRTUALENVWRAPPER_PYTHON" -c "import os,sys; sys.stdout.write(os.path.realpath(os.path.expanduser(os.path.expandvars(\"$1\")))+'\n')"
-            return 0
-        fi
+    if [ "$1" = "" ]; then
+        return 1
+    else
+        "$VIRTUALENVWRAPPER_PYTHON" -c "import os,sys; sys.stdout.write(os.path.normpath(os.path.expanduser(os.path.expandvars(\"$1\")))+'\n')"
+        return 0
+    fi
 }
 
 function virtualenvwrapper_derive_workon_home {
@@ -93,7 +122,7 @@ function virtualenvwrapper_derive_workon_home {
 
     # If the path is relative, prefix it with $HOME
     # (note: for compatibility)
-    if echo "$workon_home_dir" | (unset GREP_OPTIONS; \grep '^[^/~]' > /dev/null)
+    if echo "$workon_home_dir" | (unset GREP_OPTIONS; command \grep '^[^/~]' > /dev/null)
     then
         workon_home_dir="$HOME/$WORKON_HOME"
     fi
@@ -102,7 +131,7 @@ function virtualenvwrapper_derive_workon_home {
     # path might contain stuff to expand.
     # (it might be possible to do this in shell, but I don't know a
     # cross-shell-safe way of doing it -wolever)
-    if echo "$workon_home_dir" | (unset GREP_OPTIONS; \egrep '([\$~]|//)' >/dev/null)
+    if echo "$workon_home_dir" | (unset GREP_OPTIONS; command \egrep '([\$~]|//)' >/dev/null)
     then
         # This will normalize the path by:
         # - Removing extra slashes (e.g., when TMPDIR ends in a slash)
@@ -135,13 +164,19 @@ function virtualenvwrapper_verify_workon_home {
 
 #HOOK_VERBOSE_OPTION="-q"
 
+# Function to wrap mktemp so tests can replace it for error condition
+# testing.
+function virtualenvwrapper_mktemp {
+    command \mktemp "$@"
+}
+
 # Expects 1 argument, the suffix for the new file.
 function virtualenvwrapper_tempfile {
     # Note: the 'X's must come last
     typeset suffix=${1:-hook}
     typeset file
 
-    file="`\mktemp -t virtualenvwrapper-$suffix-XXXXXXXXXX`"
+    file="$(virtualenvwrapper_mktemp -t virtualenvwrapper-$suffix-XXXXXXXXXX)"
     if [ $? -ne 0 ] || [ -z "$file" ] || [ ! -f "$file" ]
     then
         echo "ERROR: virtualenvwrapper could not create a temporary file name." 1>&2
@@ -161,7 +196,7 @@ function virtualenvwrapper_run_hook {
     if [ -z "$VIRTUALENVWRAPPER_LOG_DIR" ]
     then
         echo "ERROR: VIRTUALENVWRAPPER_LOG_DIR is not set." 1>&2
-        \rm -f "$hook_script"
+        command \rm -f "$hook_script"
         return 1
     fi
     "$VIRTUALENVWRAPPER_PYTHON" -c 'from virtualenvwrapper.hook_loader import main; main()' $HOOK_VERBOSE_OPTION --script "$hook_script" "$@"
@@ -172,13 +207,13 @@ function virtualenvwrapper_run_hook {
         if [ ! -f "$hook_script" ]
         then
             echo "ERROR: virtualenvwrapper_run_hook could not find temporary file $hook_script" 1>&2
-            \rm -f "$hook_script"
+            command \rm -f "$hook_script"
             return 2
         fi
         # cat "$hook_script"
         source "$hook_script"
-	elif [ "${1}" = "initialize" ]
-	then
+    elif [ "${1}" = "initialize" ]
+    then
         cat - 1>&2 <<EOF 
 virtualenvwrapper.sh: There was a problem running the initialization hooks. 
 
@@ -188,7 +223,7 @@ VIRTUALENVWRAPPER_PYTHON=$VIRTUALENVWRAPPER_PYTHON and that PATH is
 set properly.
 EOF
     fi
-    \rm -f "$hook_script"
+    command \rm -f "$hook_script"
     return $result
 }
 
@@ -255,23 +290,34 @@ function virtualenvwrapper_initialize {
     return 0
 }
 
-
-# Verify that virtualenv is installed and visible
-function virtualenvwrapper_verify_virtualenv {
-    typeset venv=$(\which "$VIRTUALENVWRAPPER_VIRTUALENV" | (unset GREP_OPTIONS; \grep -v "not found"))
-    if [ "$venv" = "" ]
+# Verify that the passed resource is in path and exists
+function virtualenvwrapper_verify_resource {
+    typeset exe_path=$(command \which "$1" | (unset GREP_OPTIONS; command \grep -v "not found"))
+    if [ "$exe_path" = "" ]
     then
-        echo "ERROR: virtualenvwrapper could not find $VIRTUALENVWRAPPER_VIRTUALENV in your path" >&2
+        echo "ERROR: virtualenvwrapper could not find $1 in your path" >&2
         return 1
     fi
-    if [ ! -e "$venv" ]
+    if [ ! -e "$exe_path" ]
     then
-        echo "ERROR: Found $VIRTUALENVWRAPPER_VIRTUALENV in path as \"$venv\" but that does not exist" >&2
+        echo "ERROR: Found $1 in path as \"$exe_path\" but that does not exist" >&2
         return 1
     fi
     return 0
 }
 
+
+# Verify that virtualenv is installed and visible
+function virtualenvwrapper_verify_virtualenv {
+    virtualenvwrapper_verify_resource $VIRTUALENVWRAPPER_VIRTUALENV
+}
+
+
+function virtualenvwrapper_verify_virtualenv_clone {
+    virtualenvwrapper_verify_resource $VIRTUALENVWRAPPER_VIRTUALENV_CLONE
+}
+
+
 # Verify that the requested environment exists
 function virtualenvwrapper_verify_workon_environment {
     typeset env_name="$1"
@@ -378,7 +424,7 @@ function mkvirtualenv {
     virtualenvwrapper_verify_virtualenv || return 1
     (
         [ -n "$ZSH_VERSION" ] && setopt SH_WORD_SPLIT
-        \cd "$WORKON_HOME" &&
+        virtualenvwrapper_cd "$WORKON_HOME" &&
         "$VIRTUALENVWRAPPER_VIRTUALENV" $VIRTUALENVWRAPPER_VIRTUALENV_ARGS "$@" &&
         [ -d "$WORKON_HOME/$envname" ] && \
             virtualenvwrapper_run_hook "pre_mkvirtualenv" "$envname"
@@ -439,16 +485,16 @@ function rmvirtualenv {
         # Move out of the current directory to one known to be
         # safe, in case we are inside the environment somewhere.
         typeset prior_dir="$(pwd)"
-        \cd "$WORKON_HOME"
+        virtualenvwrapper_cd "$WORKON_HOME"
 
         virtualenvwrapper_run_hook "pre_rmvirtualenv" "$env_name"
-        \rm -rf "$env_dir"
+        command \rm -rf "$env_dir"
         virtualenvwrapper_run_hook "post_rmvirtualenv" "$env_name"
 
         # If the directory we used to be in still exists, move back to it.
         if [ -d "$prior_dir" ]
         then
-            \cd "$prior_dir"
+            virtualenvwrapper_cd "$prior_dir"
         fi
     done
 }
@@ -459,9 +505,7 @@ function virtualenvwrapper_show_workon_options {
     # NOTE: DO NOT use ls here because colorized versions spew control characters
     #       into the output list.
     # echo seems a little faster than find, even with -depth 3.
-    (\cd "$WORKON_HOME"; for f in */$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate; do echo $f; done) 2>/dev/null | \sed 's|^\./||' | \sed "s|/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate||" | \sort | (unset GREP_OPTIONS; \egrep -v '^\*$')
-
-#    (\cd "$WORKON_HOME"; find -L . -depth 3 -path '*/bin/activate') | sed 's|^\./||' | sed 's|/bin/activate||' | sort
+    (virtualenvwrapper_cd "$WORKON_HOME"; for f in */$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate; do echo $f; done) 2>/dev/null | command \sed 's|^\./||' | command \sed "s|/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate||" | command \sort | (unset GREP_OPTIONS; command \egrep -v '^\*$')
 }
 
 function _lsvirtualenv_usage {
@@ -708,14 +752,14 @@ function cdsitepackages {
     virtualenvwrapper_verify_workon_home || return 1
     virtualenvwrapper_verify_active_environment || return 1
     typeset site_packages="`virtualenvwrapper_get_site_packages_dir`"
-    \cd "$site_packages"/$1
+    virtualenvwrapper_cd "$site_packages"/$1
 }
 
 # Does a ``cd`` to the root of the currently-active virtualenv.
 function cdvirtualenv {
     virtualenvwrapper_verify_workon_home || return 1
     virtualenvwrapper_verify_active_environment || return 1
-    \cd $VIRTUAL_ENV/$1
+    virtualenvwrapper_cd $VIRTUAL_ENV/$1
 }
 
 # Shows the content of the site-packages directory of the currently-active
@@ -752,50 +796,71 @@ function toggleglobalsitepackages {
 
 # Duplicate the named virtualenv to make a new one.
 function cpvirtualenv {
-    typeset env_name="$1"
-    if [ "$env_name" = "" ]
-    then
-        virtualenvwrapper_show_workon_options
+    virtualenvwrapper_verify_workon_home || return 1
+    virtualenvwrapper_verify_virtualenv_clone || return 1
+
+    typeset src_name="$1"
+    typeset trg_name="$2"
+    typeset src
+    typeset trg 
+
+    # without a source there is nothing to do
+    if [ "$src_name" = "" ]; then
+        echo "Please provide a valid virtualenv to copy."
         return 1
-    fi
-    typeset new_env="$2"
-    if [ "$new_env" = "" ]
-    then
-        echo "Please specify target virtualenv"
-        return 1
-    fi
-    if echo "$WORKON_HOME" | (unset GREP_OPTIONS; \grep "/$" > /dev/null)
-    then
-        typeset env_home="$WORKON_HOME"
     else
-        typeset env_home="$WORKON_HOME/"
+        # see if its already in workon
+        if [ ! -e "$WORKON_HOME/$src_name" ]; then
+            # so its a virtualenv we are importing
+            # make sure we have a full path
+            # and get the name
+            src=$(virtualenvwrapper_expandpath "$src_name")
+            # final verification
+            if [ ! -e "$src" ]; then
+                echo "Please provide a valid virtualenv to copy."
+                return 1
+            fi
+            src_name=$(basename "$src")
+        else
+           src="$WORKON_HOME/$src_name"
+        fi
     fi
-    typeset source_env="$env_home$env_name"
-    typeset target_env="$env_home$new_env"
 
-    if [ ! -e "$source_env" ]
-    then
-        echo "$env_name virtualenv doesn't exist"
+    if [ "$trg_name" = "" ]; then
+        # target not given, assume
+        # same as source
+        trg="$WORKON_HOME/$src_name"
+        trg_name="$src_name"
+    else
+        trg="$WORKON_HOME/$trg_name"
+    fi
+    trg=$(virtualenvwrapper_expandpath "$trg")
+
+    # validate trg does not already exist
+    # catch copying virtualenv in workon home
+    # to workon home
+    if [ -e "$trg" ]; then
+        echo "$trg_name virtualenv already exists."
         return 1
     fi
 
-    \cp -r "$source_env" "$target_env"
-    for script in $( \ls $target_env/$VIRTUALENVWRAPPER_ENV_BIN_DIR/* )
-    do
-        newscript="$script-new"
-        \sed "s|$source_env|$target_env|g" < "$script" > "$newscript"
-        \mv "$newscript" "$script"
-        \chmod a+x "$script"
-    done
+    echo "Copying $src_name as $trg_name..."
+    (
+        [ -n "$ZSH_VERSION" ] && setopt SH_WORD_SPLIT 
+        virtualenvwrapper_cd "$WORKON_HOME" &&
+        "$VIRTUALENVWRAPPER_VIRTUALENV_CLONE" "$src" "$trg" 
+        [ -d "$trg" ] && 
+            virtualenvwrapper_run_hook "pre_cpvirtualenv" "$src" "$trg_name" &&
+            virtualenvwrapper_run_hook "pre_mkvirtualenv" "$trg_name"
+    )
+    typeset RC=$?
+    [ $RC -ne 0 ] && return $RC
 
-    "$VIRTUALENVWRAPPER_VIRTUALENV" "$target_env" --relocatable
-    \sed "s/VIRTUAL_ENV\(.*\)$env_name/VIRTUAL_ENV\1$new_env/g" < "$source_env/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate" > "$target_env/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate"
+    [ ! -d "$WORKON_HOME/$trg_name" ] && return 1
+
+    # Now activate the new environment
+    workon "$trg_name"
 
-    (\cd "$WORKON_HOME" && (
-        virtualenvwrapper_run_hook "pre_cpvirtualenv" "$env_name" "$new_env";
-        virtualenvwrapper_run_hook "pre_mkvirtualenv" "$new_env"
-        ))
-    workon "$new_env"
     virtualenvwrapper_run_hook "post_mkvirtualenv"
     virtualenvwrapper_run_hook "post_cpvirtualenv"
 }
